﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.SharePoint;
using Telerik.JustMock;
using System.Diagnostics;

namespace JustMockSharePointSamples.ComponentsTests
{
	/// <summary>
	/// Summary description for UnitTest1
	/// </summary>
	[TestClass]
	public class SPContextTests
	{
		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		[TestMethod]
		[ExpectedException(typeof(ApplicationException))]
		public void Current_NotAvailable_ThrowsApplicationException()
		{
			// Arrange
			Mock.Arrange(() => SPContext.Current).Throws(new ApplicationException("Not allowed to call SPContext.Current"));

			// Act
			SPContext currentContext = SPContext.Current;
		}
		
		[TestMethod]
		public void Current_CurrentContextIsFake_ReturnsFakeSPContext()
		{
			// Arrange
			var fakeContext = Mock.Create<SPContext>();
			Mock.Arrange(() => SPContext.Current).Returns(fakeContext);

			// Act
			SPContext currentContext = SPContext.Current;

			// Assert
			Assert.IsNotNull(currentContext, "The current SPContext should not be null");
		}

		[TestMethod]
		public void SPWeb_AllowAnonymousAccess_ReturnsTrue()
		{
			// Arrange
			var fakeContext = Mock.Create<SPContext>();
			Mock.Arrange(() => fakeContext.Web.AllowAnonymousAccess).Returns(true);

			// Assert
			Assert.IsTrue(fakeContext.Web.AllowAnonymousAccess, "Our SPWeb should allow anonymous access");
		}

		[TestMethod]
		public void CurrentWeb_AllowAnonymousAccess_ReturnsTrue()
		{
			// Arrange
			Mock.Arrange(() => SPContext.Current.Web.AllowAnonymousAccess).Returns(true);

			// Assert
			Assert.IsTrue(SPContext.Current.Web.AllowAnonymousAccess, "Anonymous access should be allowed on our current SPWeb");
		}

		[TestMethod]
		public void CurrentWeb_VerboseSetupAllowAnonymousAccess_ReturnsTrue()
		{
			// Arrange
			var fakeContext = Mock.Create<SPContext>();
			Mock.Arrange(() => SPContext.Current).Returns(fakeContext);

			var fakeWeb = Mock.Create<SPWeb>();
			Mock.Arrange(() => fakeContext.Web).Returns(fakeWeb);
			Mock.Arrange(() => fakeWeb.AllowAnonymousAccess).Returns(true);

			// Assert
			Assert.IsTrue(SPContext.Current.Web.AllowAnonymousAccess, "Anonymous access should be allowed on our current SPWeb");
		}


        [TestMethod]
        public void Should_Assert_FakingOfSpList()
        {
            var spWeb = Mock.Create<SPWeb>();
            var spList = Mock.Create<SPList>();

            var spListCollection = Mock.Create<SPListCollection>();
            var spListItemCollection = Mock.Create<SPListItemCollection>();

            Mock.Arrange(() => spWeb.Lists).Returns(spListCollection);
            Mock.Arrange(() => spListCollection[Arg.AnyString]).Returns(spList);

            Mock.Arrange(() => spList.GetItems(Arg.IsAny<SPQuery>())).Returns(spListItemCollection);

            Assert.AreEqual(spListCollection, spWeb.Lists);
            Assert.AreEqual(spList, spWeb.Lists["myList"]);
            Assert.AreEqual(spListItemCollection, spWeb.Lists["myList"].GetItems(new SPQuery()));
        }

	}
}
